#!/usr/bin/env python
# Python BySerial www.byvac.com
# based on
# Very simple serial terminal by # Chris Liechti
#
from __future__ import print_function
# This is a modified terminal program that is line oriented rather than the
# normal character orientation. The main advantage is that the built in
# history provided by Python can be used.
# No GUI has been used so that it can be used over a remote terminal and will
# also be cross platform.
# Dependancies:
#  * pyserial
#  * colorama
#
# Version 02  December 2013
#   Does not use command line options because it can now be supplied as a
#   single exe file. Extended so that the settings can be saved in bvs.cfg
# Version 04 Jan 2014
#   Removed settings
# version 5
#version 11 added boot loader

VERSION = '12'
CONFFILE = 'bvterm'+VERSION+'.cfg'

import sys, os, serial, threading, bvtools
from serial.urlhandler import protocol_socket # this is so pyInstaller can find it
from time import sleep
import subprocess # to call command line

# ------------------ Terminal colour -------------------------------------------
# Windows does not do colour natively and so needs colorama
if os.name == 'nt':
    # colorama imports
    import fixpath
    from colorama import init, Fore, Back, Style
    init() # colorama initialisation for windows, Linux ignores
else:
    class AnsiStyle:
        BRIGHT = '\x1b[1m'
        DIM = '\x1b[2m'
        NORMAL = '\x1b[22m'
        RESET_ALL = '\x1b[0m'
    class AnsiFore:
        BLACK = '\x1b[30m'
        RED = '\x1b[31m'
        GREEN = '\x1b[32m'
        YELLOW = '\x1b[33m'
        BLUE = '\x1b[34m'
        MAGENTA = '\x1b[35m'
        CYAN = '\x1b[36m'
        WHITE = '\x1b[37m'
        RESET = '\x1b[39m'
    class AnsiBack:
        BLACK = '\x1b[40m'
        RED = '\x1b[41m'
        GREEN = '\x1b[42m'
        YELLOW = '\x1b[43m'
        BLUE = '\x1b[44m'
        MAGENTA = '\x1b[45m'
        CYAN = '\x1b[46m'
        WHITE = '\x1b[47m'
        RESET = '\x1b[49m'

    Fore = AnsiFore
    Back = AnsiBack
    Style =  AnsiStyle

AvailablePorts = []
try:
    from serial.tools.list_ports import comports
except ImportError:
    comports = None
else:
    for port, desc, hwid in sorted(comports()):
        AvailablePorts.append([port,desc])




DEFAULT_PORT = None
DEFAULT_BAUDRATE = 9600
DEFAULT_RTS = None
DEFAULT_DTR = False
COL_BACKGROUND = Back.BLACK
COL_HOST = [Style.NORMAL, Fore.WHITE, COL_BACKGROUND] # normal colour
COL_COMSIN = [Style.BRIGHT, Fore.YELLOW, COL_BACKGROUND] #colour from host
COL_ESC = [Fore.CYAN, COL_BACKGROUND]
COL_INFO = [Fore.GREEN, COL_BACKGROUND]
COL_ERROR = [Fore.RED, COL_BACKGROUND]
COL_HELP = [Fore.CYAN, COL_BACKGROUND]

if sys.version_info >= (3, 0):
    def character(b):
        return b.decode('latin1')
else:
    def character(b):
        return b

# Some colour functions
def set_col(colour):
    for n in colour:
        sys.stdout.write(n)

def cprint(colour,data):
    set_col(colour)
    sys.stdout.write(data)
    set_col(COL_HOST) # reset

def get_help_text():
    return """
--- BySerial Help (www.byvac.com), Version \'%s\'
--- Commands start with a dot in the first character position
--- .h [x] Help more help if x = cmd as below, e.g. .h[esc]
--- .x  Exit
--- Console Commands:
---    '.port [n]' Change port, supply name optional
---    '.baud n' Change baud rate
---    '.data n' Set number of data bits
---    '.stop n' Set number of stop bits
---    '.esc n'  Where nis 0 to 3 Shows None=0 All = 4
---    '.parity x' x=[o,e,m,s,n] Set parity
---    '.stop n' 1=1, 2=2, 3=1.5
---    '.flow [n,x,h]' x =xon/xoff, h=hardware, n=none
---    '.lf or .cr or .crlf sets end of line char or combination 
--- Toggles:
---    '.rts' Sets/Resets  '.cts' Sets/Resets
---    '.brk' Sets/Resets
---    '.info' Show Info
--- BV Options:
---    '.r'    Toggle DTR (Reset) (ByPic)
---    '.tl'   Text load see .h tl (ByPic)
---    '.sv3 [x]' SV3 commands .h sv3 for options (PiChips)
---    '.upm'  Upload media filem .bmp or .bff file or directory (BV501)
---    '.upf'  Uploads file or directory to SD Card on BV5nn
---    '.bl   Boot loader
---    '.loopbreak' - clears flash, thus gettingoutof an endless main() loop
---    '.#' Sends number instead of charater - see .h .# 
""" % (VERSION)

def help_esc():
    return """
Escape has three modes:
    0) No control characters are shown
    1) Control characters are shown but not RT or LF
    3) All control characters are shown
    4) All out put from the target is shown as hex
    A control charatcer is a character with a value below 32, example:
    .esc 4 -- sets out put to be hex, useful for debugging
"""

def help_sv3():
    return """
    Command options for sv3:
    .sv3 set - sets debug level 1 so thet the [6] can be seen and
         also sends just CR when <ent> is pressed
    .sv3 list - Lists all connected devices
    .sv3 eeread adr - Lists contents of eerpm for device with
         specified address 'adr' e.g. .sv3 eeread h
    .sv3 eewrite adr location value
         writes to eeprom of device with 'adr'. e.g. to write
         55 to eeprom location 200 of device 'h' then:
         .sv3 eewrite h 200 55
"""

def help_tl(self):
    return """
    Text upload to a ByPic device.
    .tl opens a dialog so file can be chosen and sent
    .tll will send the previous file chosen with .tl
    Also enables sending start string if required.

    Current file: \'%s\'
    Current start string is \'%s\'
""" % (self.cf.filename,self.cf.start)

def help_upm():
    return """
    Upload a media (bmp or bff) file or directory
    Options: .upm  or  .upm dir
    Choose file or directory from dialog.
    Radio buttons:
    No Dup:  Duplicate files will be ignored
    Dup:     Duplicate files will be stored
    Del Dup: File stored and duplicate names deleted
"""

def help_dothash():
    return """
    .# Help
    It is sometimes necessary to send a byte value to a device rather
    than its charatcer representation. For example typing 6 at the terminal
    will in fact send the byte value 54 which is the ASCII code for '6'. If 
    you want to atcually send 6 then CTRL f would do it or at the terminal 
    put in .#6 - any number after .# will be sent as that number. 
    This can be combined so sending 1 2 3 'fred' would be .#1.#2.#3 fred
"""

# Individual help modes
def help_cmd(self,cmd):
    if 'esc' in cmd:
        cprint(COL_HELP, help_esc())
    if 'tl' in cmd:
        cprint(COL_HELP, help_tl(self))
    if 'sv3' in cmd:
        cprint(COL_HELP, help_sv3())
    if 'upm' in cmd:
        cprint(COL_HELP, help_upm())
    if '.#' in cmd:
        cprint(COL_HELP, help_dothash())

def dump_port_list():
    if comports:
        cprint(COL_INFO, '\n--- Available ports:\n')
        for port, desc, hwid in sorted(comports()):
            cprint(COL_INFO, '--- %-20s %s\n' % (port, desc))


CONVERT_CRLF = 2
CONVERT_CR   = 1
CONVERT_LF   = 0
NEWLINE_CONVERISON_MAP = ('\n', '\r', '\r\n')
LF_MODES = ('LF', 'CR', 'CR/LF')

REPR_MODES = ('raw', 'some control', 'all control', 'hex')

class Miniterm(object):
    def __init__(self, config):
        self.cf = config # class instance for settings
        try:
            self.serial = serial.serial_for_url(self.cf.port, self.cf.baudrate, parity=self.cf.parity, rtscts=self.cf.RtsCts, xonxoff=self.cf.XonXoff, timeout=.001)
        except AttributeError:
            # happens when the installed pyserial is older than 2.5. use the
            # Serial class directly then.
            self.serial = serial.Serial(self.cf.port, self.cf.baudrate, parity=self.cf.parity, rtscts=self.cf.RtsCts, xonxoff=self.cf.XonXoff, timeout=1)
        self.newline = NEWLINE_CONVERISON_MAP[self.cf.outgoing]
        self.break_state = False

    def _start_reader(self):
        """Start reader thread"""
        self._reader_alive = True
        self.receiver_thread = threading.Thread(target=self.reader)
        self.receiver_thread.setDaemon(True)
        self.receiver_thread.start()

    def _stop_reader(self):
        """Stop reader thread only, wait for clean exit of thread"""
        self._reader_alive = False
        self.receiver_thread.join()


    def start(self):
        self.alive = True
        self._start_reader()
        self.transmitter_thread = threading.Thread(target=self.writer)
        self.transmitter_thread.setDaemon(True)
        self.transmitter_thread.start()

    def stop(self):
        self.alive = False

    def join(self, transmit_only=False):
        self.transmitter_thread.join()
        if not transmit_only:
            self.receiver_thread.join()

    def dump_port_settings(self):
        cprint(COL_INFO, "\n--- Settings: %s  %s,%s,%s,%s\n" % (
                self.serial.portstr,
                self.serial.baudrate,
                self.serial.bytesize,
                self.serial.parity,
                self.serial.stopbits))
        cprint(COL_INFO,'--- RTS: %-8s  DTR: %-8s  BREAK: %-8s\n' % (
                (self.cf.RTS and 'active' or 'inactive'),
                (self.cf.DTR and 'active' or 'inactive'),
                (self.break_state and 'active' or 'inactive')))
        try:
            cprint(COL_INFO, '--- CTS: %-8s  DSR: %-8s  RI: %-8s  CD: %-8s\n' % (
                    (self.serial.getCTS() and 'active' or 'inactive'),
                    (self.serial.getDSR() and 'active' or 'inactive'),
                    (self.serial.getRI() and 'active' or 'inactive'),
                    (self.serial.getCD() and 'active' or 'inactive')))
        except serial.SerialException:
            # on RFC 2217 ports it can happen to no modem state notification was
            # yet received. ignore this error.
            pass
        cprint(COL_INFO, '--- software flow control: %s\n' % (self.serial.xonxoff and 'active' or 'inactive'))
        cprint(COL_INFO, '--- hardware flow control: %s\n' % (self.serial.rtscts and 'active' or 'inactive'))
        cprint(COL_INFO, '--- data escaping: %s  linefeed: %s\n' % (
                REPR_MODES[self.cf.escmode],
                LF_MODES[self.cf.outgoing]))
        
    def reader(self):
        """loop and copy serial->console"""
        try:
            while self.alive and self._reader_alive:
                data = character(self.serial.read(1))
                if len(data) > 0:

                    set_col(COL_COMSIN)
                    if self.cf.escmode == 0:
                        # direct output, just have to care about newline setting
                        if data == '\n' and self.cf.outgoing == CONVERT_LF:
                            sys.stdout.write('\n')
                        elif data == '\r' and self.cf.outgoing == CONVERT_CR:
                            sys.stdout.write('\n')
                        elif data == '\t':
                            sys.stdout.write('    ')
                        elif ord(data) > 31:
                            sys.stdout.write(data)
                    elif self.cf.escmode == 1:
                        # escape non-printable, let pass newlines
                        if self.cf.outgoing == CONVERT_CRLF and data in '\r\n':
                            if data == '\n':
                                sys.stdout.write('\n')
                            elif data == '\r':
                                pass
                        elif data == '\n' and self.cf.outgoing == CONVERT_LF:
                            sys.stdout.write('\n')
                        elif data == '\r' and self.cf.outgoing == CONVERT_CR:
                            sys.stdout.write('\n')
                        elif data == '\t':
                            sys.stdout.write('    ')
                        else:
                            for n in data:
                                if ord(n) < 32: cprint(COL_ESC, '[%x]' % ord(n))
                                else: cprint(COL_COMSIN, n)
                    elif self.cf.escmode == 2:
                        # escape all non-printable, including newline
                        for n in data:
                            if ord(n) < 32: cprint(COL_ESC, '[%x]' % ord(n))
                            else: cprint(COL_COMSIN, n)
                    elif self.cf.escmode == 3:
                        # escape everything (hexdump)
                        for c in data:
                            sys.stdout.write("%s " % c.encode('hex'))
                    sys.stdout.flush()
                    set_col(COL_HOST)
        except serial.SerialException, e:
            self.alive = False
            # would be nice if the console reader could be interruptted at this
            # point...
            raise

    # dot must be followed by # so that pic.bmpwill get through [1]
    def processHash(self, s):
        # eg s = 'rc .#40.#41.#6.#6 7 22 fred'
        inhash = False
        indot = False
        acc = ''
        str = ''
        for c in s:
            if c == '.':
                indot = True
                continue # dont get dots
            if (c.isdigit() == False and inhash == True):
                str += chr(int(acc))
                acc = ''
                inhash = False
            if c == '#' and indot: 
                inhash = True
                indot = False
                continue
            elif indot:   #[1]
                tmp = '.'+c
                c = tmp
                indot = False
            if inhash:
                acc += c
                continue       
            str += c
        if len(acc):        
            str += chr(int(acc))        
        return str    

    def writer(self):
        """\
        This is the main loop
        """
        try:
            while self.alive:
                s = raw_input()
                if s == '.x':    
                    self.stop()
                    self.cf.write()
                    break
                elif s[0:2] == '.h':
                    if len(s) > 2:
                        help_cmd(self,s)
                    else:
                        cprint(COL_INFO, get_help_text())
                elif s[0:1] == '.':
                    self.main_menu(s)
                else:
                    self.serial.write(self.processHash(s))       # send bytes
                    #self.serial.write(s)
                    self.serial.write(self.newline)         # send newline character(s)

        except:
            self.alive = False
            raise


    def main_menu(self,s):
        """
        Handles all console commands refering to the com port and how the
        terminal works, such as line feed, baudrate etc.
        """
        if '.rts' in s:                       # CTRL+R -> Toggle RTS
            self.cf.RTS = not self.cf.RTS
            self.serial.setRTS(self.cf.RTS)
            cprint(COL_INFO, '--- RTS %s ---\n' % (self.cf.RTS and 'active' or 'inactive'))
        elif '.dtr' in s:                       # CTRL+D -> Toggle DTR
            self.cf.DTR = not self.cf.DTR
            self.serial.setDTR(self.cf.DTR)
            cprint(COL_INFO, '--- DTR %s ---\n' % (self.cf.DTR and 'active' or 'inactive'))
        elif '.brk' in s:                       # CTRL+B -> toggle BREAK condition
            self.break_state = not self.break_state
            self.serial.setBreak(self.break_state)
            cprint(COL_INFO, '--- BREAK %s ---\n' % (self.break_state and 'active' or 'inactive'))
        elif '.inf' in s:                       # CTRL+I -> info
            self.dump_port_settings()
        elif '.esc' in s:                       # CTRL+A -> cycle escape mode
            m = int(s[4:])
            if m > 3: m = 3
            self.cf.escmode = m
            cprint(COL_INFO, '--- escape data: %s ---\n' % ( REPR_MODES[self.cf.escmode],))
        elif s == '.crlf':                       # CRLF
            self.cf.outgoing = 2
            self.newline = NEWLINE_CONVERISON_MAP[self.cf.outgoing]
            cprint(COL_INFO, '--- line feed %s ---\n' % (
                LF_MODES[self.cf.outgoing],
            ))
        elif s == '.lf':
            self.cf.outgoing = 0
            self.newline = NEWLINE_CONVERISON_MAP[self.cf.outgoing]
            cprint(COL_INFO, '--- line feed %s ---\n' % (
                LF_MODES[self.cf.outgoing],
            ))
        elif s == '.cr':    
            self.cf.outgoing = 1
            self.newline = NEWLINE_CONVERISON_MAP[self.cf.outgoing]
            cprint(COL_INFO, '--- line feed %s ---\n' % (
                LF_MODES[self.cf.outgoing],
            ))
        elif '.port' in s:                         # P -> change port
            if len(s) > 5:
                port = s[5:]
            else:
                dump_port_list()
                try:
                    port = sys.stdin.readline().strip()
                except KeyboardInterrupt:
                    port = None
            if port and port != self.serial.port:
                # reader thread needs to be shut down
                self._stop_reader()
                # save settings
                settings = self.serial.getSettingsDict()
                try:
                    try:
                        new_serial = serial.serial_for_url(port, do_not_open=True)
                    except AttributeError:
                        # happens when the installed pyserial is older than 2.5. use the
                        # Serial class directly then.
                        new_serial = serial.Serial()
                        new_serial.port = port
                    # restore settings and open
                    new_serial.applySettingsDict(settings)
                    new_serial.open()
                    new_serial.setRTS(self.cf.RTS)
                    new_serial.setDTR(self.cf.DTR)
                    new_serial.setBreak(self.break_state)
                except Exception, e:
                    cprint(COL_ERROR, '--- ERROR opening new port: %s ---\n' % (e,))
                    new_serial.close()
                else:
                    self.serial.close()
                    self.serial = new_serial
                    self.cf.port = port # so config file is updated
                    cprint(COL_INFO, '--- Port changed to: %s ---\n' % (self.serial.port,))
                # and restart the reader thread
                self._start_reader()
        elif '.baud' in s:               # B -> change baudrate
            sys.stdout.flush()
            backup = self.serial.baudrate
            if len(s) > 5:
                newRate = int(s[5:].strip())
            else:
                cprint(COL_INFO, '\n--- Baudrate: ')
                newRate = int(sys.stdin.readline().strip())
            try:
                self.serial.baudrate = newRate
            except ValueError, e:
                cprint(COL_ERROR, '--- ERROR setting baudrate: %s ---\n' % (e,))
                self.serial.baudrate = backup
            else:
                self.dump_port_settings()
                self.cf.baudrate = newRate
        elif '.data' in s:
            bits = int(s[5:])
            if bits == 8:                          # 8 -> change to 8 bits
                self.serial.bytesize = serial.EIGHTBITS
                self.dump_port_settings()
            elif bits == 7:                          # 7 -> change to 8 bits
                self.serial.bytesize = serial.SEVENBITS
                self.dump_port_settings()
        elif '.parity' in s:
            c = s[7:]
            if c in 'eE':                         # E -> change to even parity
                self.serial.parity = serial.PARITY_EVEN
                self.cf.parity = 'E'
                self.dump_port_settings()
            elif c in 'oO':                         # O -> change to odd parity
                self.serial.parity = serial.PARITY_ODD
                self.cf.parity = 'O'
                self.dump_port_settings()
            elif c in 'mM':                         # M -> change to mark parity
                self.serial.parity = serial.PARITY_MARK
                self.cf.parity = 'M'
                self.dump_port_settings()
            elif c in 'sS':                         # S -> change to space parity
                self.serial.parity = serial.PARITY_SPACE
                self.cf.parity = 'S'
                self.dump_port_settings()
            elif c in 'nN':                         # N -> change to no parity
                self.serial.parity = serial.PARITY_NONE
                self.cf.parity = 'N'
                self.dump_port_settings()
            elif c == '1':                          # 1 -> change to 1 stop bits
                self.serial.stopbits = serial.STOPBITS_ONE
            self.dump_port_settings()
        elif '.stop' in s:
            m = int(s[5:].strip())
            if m == 2:                          # 2 -> change to 2 stop bits
                self.serial.stopbits = serial.STOPBITS_TWO
                self.dump_port_settings()
            elif m == 3:                          # 3 -> change to 1.5 stop bits
                self.serial.stopbits = serial.STOPBITS_ONE_POINT_FIVE
                self.dump_port_settings()
        elif '.flow' in s:
            c = s[5:]
            self.serial.xonxoff = False
            self.serial.rtscts = False
            self.cf.XonXoff = False
            self.cf.RtsCts = False
            if 'xX' in c:
                self.serial.xonxoff = True
                self.cf.XonXoff = True
            if 'hH' in c:
                self.serial.rtscts = True
                self.cf.RtsCts = True
            self.dump_port_settings()
        ### ByVac commands
        elif s == '.r': # reset
            self.serial.setDTR(True)
            sleep(.1)
            self.serial.setDTR(False)
        ### Text load with dialog
        elif s == '.tl':
            # loads user filename and sends
            from tload import tload
            #opens a dialog box andreturns result as a list res[]
            gFile = tload("Get File",[self.cf.filename,self.cf.start])
            res = gFile.result()  # this is a 2 part list
            self.cf.filename = res[0]
            self.cf.start = res[1]
            if res[2] == '1': # this will be '0' if cancelled pressed
                self._stop_reader() # reader thread will interfere with transfer
                bvtools.tload_transfer(self.serial,self.cf.start,self.cf.filename)
                self._start_reader()
                self.cf.write() # update for next time
        elif s == '.tll':
            # send previously loaded file
            self._stop_reader() # reader thread will interfere with transfer
            bvtools.tload_transfer(self.serial,self.cf.start,self.cf.filename)
            self._start_reader()
        elif s[0:4] == '.sv3':
            self._stop_reader() # reader thread will interfere with transfer
            if 'list' in s:
                bvtools.sv3_list(self.serial)
            elif 'eeread' in s:
                adr = s.split()[2]
                if len(adr) > 0:
                    bvtools.sv3_eeRead(self.serial,adr)
                else:
                    sys.stdout.write('\nSupply address, e.g. .sv3 eeread h\n')
            elif 'eewrite' in s: # .sv3 eewrite h 27 33
                tmp = s.split()
                bvtools.sv3_eeWrite(self.serial,tmp[2],tmp[3],tmp[4])
            elif 'set' in s:
                sys.stdout.write('\nSV3 Parameters set "-D --cr"\n')
                self.cf.escmode = 1 # -D
                self.cf.outgoing = 1 # --cr
            else:
                sys.stdout.write('\nExspession incorect see help .h sv3\n')
            self._start_reader()
        elif s[0:4] == '.upm':
            # version 10 upm|upm dir
            from upm import upm #dialog
            tmp = s.split()
            if len(tmp) == 2:
                import os.path
                path, filename = os.path.split(self.cf.upmname)
                gFile = upm("Get File",[path,self.cf.upmdup,'dir'])
            else:
                gFile = upm("Get File",[self.cf.upmname,self.cf.upmdup,'file'])
            res = gFile.result()  # this is a 2 part list
            #cprint(COL_INFO, 'res[0] %s\n' % (res[0]))
            #cprint(COL_INFO, 'res[1] %s\n' % (res[1]))
            self.cf.upmname = res[0]
            self.cf.upmdup = res[1]
            self._stop_reader() # reader thread will interfere with transfer
            if res[1] != 'C': # C = cancel
                bvtools.media_upload(self.serial,self.cf.upmname,self.cf.upmdup,self.cf) # file/dir upload
            self._start_reader()
        elif s[0:4] == '.upf':
            # loads user filename and sends
            from fload import fload
            gFile = fload("Get File",[self.cf.ffilename])
            res = gFile.result()  # this is a 2 part list
            self.cf.ffilename = res[0]
            if res[1] != 'C':
                self._stop_reader() # reader thread will interfere with transfer
                #if len(tmp) == 3:
                #    bvtools.file_upload(self.serial,self.cf.ffilename,2) # dir upload
                #else:
                bvtools.file_upload(self.serial,self.cf.ffilename,1) # file upload
                self._start_reader()
        elif s == '.loopbreak':
            self._stop_reader() # reader thread will interfere with transfer
            bvtools.loop_break(self.serial)
            self._start_reader()
        elif '.[' in s:
            # send numeric equivalent
            #self.serial.write(int(s[3:]))
            tmp = s.split('.[')
            #print tmp
            sys.stdout.write(tmp)
        elif '.bl' in s:
            # boot loader uses p32bl which is the library part of BV_COM1.34
            # so make sure it exists
            # requires p32bl [-v] port baud Fstart[1] Fsize[2] Delay[5] Filename[3]
            import os.path
            if os.path.isfile('p32bl.exe'):
                from bootl import bootl
                # get all details from a dialog box
                gFile = bootl("Get Image File",[self.cf.verbose,self.cf.flashStart,self.cf.flashSize,self.cf.bfileName,self.cf.eraseDelay])
                res = gFile.result()  # this is a list
                #bootString = '0'+res[0]+'1'+res[1]+'2'+res[2]+'3'+res[3]+'4'+res[4]
                #cprint(COL_INFO, '--- boot string =%s\n' % (bootString))
                if res[4] == 'G':
                    if len(res[0]) > 0:
                        self.cf.verbose = '-v'
                    else:
                        self.cf.verbose = ''
                    self.cf.flashStart = res[1]
                    self.cf.flashSize = res[2]
                    self.cf.bfileName = res[3]
                    self.cf.eraseDelay = res[5]
                    idx1 = self.cf.port.find('M')+1
                    tmp = 'p32bl '+self.cf.verbose+' '+self.cf.port[idx1:]+' '+self.cf.baudrate
                    tmp = tmp +' '+self.cf.flashStart+' '+self.cf.flashSize+' '+self.cf.eraseDelay
                    tmp = tmp +' '+self.cf.bfileName
                    cprint(COL_INFO, '--- boot string =%s\n' % (tmp))
                    # need to close com port for p32bl to use
                    self._stop_reader() # reader thread will interfere with transfer
                    self.serial.close()
                    sleep(1)
                    return_code = subprocess.call(tmp, shell=True)
                    cprint(COL_INFO, '--- return code=%d\n' % (return_code))
                    self.serial = serial.serial_for_url(self.cf.port, self.cf.baudrate, parity=self.cf.parity, rtscts=self.cf.RtsCts, xonxoff=self.cf.XonXoff, timeout=.001)
                    self._start_reader()
                    self.cf.write() # update for next time
            else:
                cprint(COL_ERROR, '\n--- Missing exe file p32bl.exe\n--- put in the same directory as the bvserial.exe')
                cprint(COL_ERROR, '\n--- p32bl.exe should be with the bvserial zip file (www.byvac.com)\n')
        elif '.edit' in s:
            from subprocess import Popen
            #if len(self.cf.filename) > 0:
            Popen(['edit/notepad++.exe',self.cf.filename])
            #else:
            #    Popen(['edit/notepad++.exe'])
        elif '.test'in s:
            self._stop_reader() # reader thread will interfere with transfer
            #bvtools.test(self.serial,'x')
            self._start_reader()
        else:
            cprint(COL_ERROR, '--- unknown menu command %s --\n' % s)


def main():
        
    from comConfig import Config
    try:
        cf = Config(CONFFILE)
    except: # must be new or corrupt config
        cprint(COL_ERROR, '--- There is a problem with the config file %s --\n' % CONFFILE)
        cprint(COL_ERROR, '--- Try deleting it and running again --\n')

    # check to see if port exists if not put user picks
    # from version 8 user always picks
    aps = ''.join(map(str, AvailablePorts))
    #if not cf.port in aps:
    dump_port_list()
    cprint(COL_INFO, '--- socket://<ipaddress>:port, e.g. socket://192.168.11.220:8080\n')
    if os.name == 'nt':
        newPort = raw_input('Enter port name:<ent> for('+cf.port+')').upper()
    else:
        newPort = raw_input('Enter port name:')
    if len(newPort) !=0:
        cf.port = newPort

    try:
        miniterm = Miniterm(cf) # all information is in a separate config file 'cf'
    except serial.SerialException, e:
        cprint(COL_ERROR, "could not open port %r: %s\n" % (cf.port, e))
        sys.exit(1)

    if not cf.quiet:
        cprint(COL_HOST, '--- BySerial on %s: %d,%s,%s,%s ---\n' % (
            miniterm.serial.portstr,
            miniterm.serial.baudrate,
            miniterm.serial.bytesize,
            miniterm.serial.parity,
            miniterm.serial.stopbits,
        ))
        cprint(COL_HOST, '--- Quit: .x  | Help: .h \n' )

    if cf.DTR is not None:
        if not cf.quiet:
            cprint(COL_INFO, '--- forcing DTR %s\n' % (cf.DTR and 'active' or 'inactive'))
        miniterm.serial.setDTR(cf.DTR )
    if cf.RTS is not None:
        if not cf.quiet:
            cprint(COL_INFO, '--- forcing RTS %s\n' % (cf.RTS and 'active' or 'inactive'))
        miniterm.serial.setRTS(cf.RTS)

    cprint(COL_INFO, '--- Verison %s\n' % (VERSION))
    miniterm.start()
    try:
        miniterm.join(True)
    except KeyboardInterrupt:
        pass
    if not cf.quiet:
        cprint(COL_HOST, "\n--- exit ---\n")
    miniterm.join()


if __name__ == '__main__':
    try:
        main()
    except:
        cprint(COL_ERROR, '--- Press any key to close --\n')
        raw_input()



